/* axdialbrain.c
 *
 * Copyright (C) 2001 RedHat, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
/*
 * Modified for SIKIGAMI 2001,2002 Axe,Inc.
 */

#include "axdialbrain.h"
#include "axdialer.h"
#include "misc.h"
#include <ctype.h>
#include <string.h>
#include <time.h>
#include <syslog.h> 
#include <malloc.h>

static char      rbrackets[] = ".-:'\">)]}"; 
static char      lbrackets[] = ".-:'\"<([{";
static char       brackets[] = ".-:'\"<>()[]{}";
static char menu_bad_words[] = "press push hit enter type key";


void
AxDialBrain_init( AxDialBrain * brain )
{
//	AxDialer_init( brain->dialer );

  AxDialBrain_reset( brain );
}

void
AxDialBrain_init_valued( AxDialBrain * brain, AxDialer * dialer )
{
	brain->dialer = dialer;
//	AxDialer_init( dialer );

  AxDialBrain_reset( brain );
}

void
AxDialBrain_clear( AxDialBrain * brain )
{
//	AxDialer_clear( brain );
}

void
AxDialBrain_reset( AxDialBrain * brain )
{
    brain->sent_login	     =  0;
    brain->prompt_tries    =  0;
    brain->prompt_response = "";
}

char *
AxDialBrain_guess_menu( AxDialBrain *brain, char * buf )
{
  char *	 cptr = NULL; 
  char *	 line; 
  char *	 nextline;
  BrainToken * tok;

  while( strstr( buf, "ppp" ) != NULL ) {
  	cptr = strstr( buf, "ppp" );
		// pick out the beginning of the line containing "ppp"
		for( line=cptr; line >= buf && !isnewline( *line ); line-- ) {}
		line++;		// too far!

		// find the beginning of the next line in the buffer, or the
		// end of the buffer... so we know where to stop our searches.
		for( nextline = line; *nextline && !isnewline( *nextline ); nextline++ )
	    {}
    
		buf = nextline; // now 'continue' will check the next line

		// Now tokenize the line and perform an IntelliSearch (tm)
		tok = AxDialBrain_tokenize( brain, line, nextline );
		if( tok ) {
	  	AxDialBrain_guess_menu_gets( brain, tok );	// may call set_prompt_response()
	  	AxDialBrain_token_list_done( brain, tok );
		}
  }

  if( cptr )
		return( cptr + 4 );	// return pointer directly AFTER "ppp".
  else
    return( NULL );
}

char *
AxDialBrain_check_prompt( AxDialBrain * brain, char * buffer )
{
  char * tprompt;
    
  // If we've been here too many times, or too long ago, just give up and
  // start pppd.
  if( brain->prompt_tries >= 5 || time( NULL ) - brain->dialer->last_rx >= 5 ) {
 // 	syslog( LOG_INFO, "Don't know what to do! Starting pppd and hoping for the best.\n" );
   	AxDialer_start_ppp( brain->dialer );

  } else if( AxDialBrain_is_login_prompt( brain, buffer ) ) {
    // We have a login prompt, so send a suitable response.
    char * send_this = brain->dialer->options.login;
//    syslog( LOG_INFO, "Looks like a login prompt.\nSending: %s\n", send_this );
   	AxDialer_reset_offset( brain->dialer );
   	brain->prompt_tries++;
		return( send_this );

  } else if( AxDialBrain_is_password_prompt( brain, buffer ) ) {
  	// We have a password prompt, so send a suitable resonse.
//   	syslog( LOG_INFO, "Looks like a password prompt.\nSending: (password)\n" );
   	AxDialer_reset_offset( brain->dialer );
   	brain->prompt_tries++;
   	brain->sent_login = 1;	// yes, we've sent a password:
   			// assume we've sent username too.
   	return( brain->dialer->options.password );

  } else if( AxDialBrain_is_welcome_msg( brain, buffer ) ) {
//   	syslog( LOG_INFO, "Looks like a welcome message.\n" );
   	AxDialer_start_ppp( brain->dialer );

  } else if( AxDialBrain_is_prompt( brain, buffer, NULL, 0 ) ) {
   	// We have some other prompt.
   	if( AxDialer_is_pending( brain->dialer ) ) {
   	  return( NULL );	// figure it out next time
    }

   	if( !brain->prompt_response[0] )
      brain->prompt_response = brain->dialer->options.default_reply;	// wild guess

//    syslog( LOG_INFO, "Hmm... a prompt.  Sending \"%s\".\n", brain->prompt_response );
    AxDialer_reset_offset( brain->dialer );
    brain->prompt_tries++;

		  // only use our prompt guess the first time.
		// if it fails, go back to the default reply.
		tprompt = brain->prompt_response;
		brain->prompt_response = "";
	
   	return( tprompt );

  } else {
   	// not a prompt at all!
   	if( AxDialer_is_pending(brain->dialer) )
   	    return( NULL );	// figure it out next time
  }

  // If we get here, then we aren't at a prompt.
  return( NULL );
}

int
AxDialBrain_is_prompt(AxDialBrain*brain,char*c,char*promptstring,int dots_wild)
{
 int foundpunct;
 char *	cptr;
 static const char *	prompt_punct	= ")>}]:.|-?$%=";
    
  // if no promptstring was given, the search is simple:  it is a 
  // prompt if the last line ends in punctuation and no newline.
  if( !promptstring ) {
	  for( cptr = c + strlen( c ) - 1; cptr >= c; cptr-- ) {
	    if( isnewline( *cptr ) ) {
				if ( !brain->prompt_response[0] )
		      return( 0 ); // last line was empty: not a prompt
		    else
		      return( 1 ); // we have a guess, so use it anyway
	    }
	    if( strchr( prompt_punct, *cptr ) )
		    return( 1 );  // first non-whitespace was punctuation! good.
	    if ( !isspace( *cptr ) )
		    return( 0 ); // not punctuation or whitespace at end
	  }
	  return( 0 );
  }
    
  // seek backwards from the end of the buffer to the beginning of
  // the last line, or the beginning of the buffer, whichever comes first.
  // Then skip leading whitespace so that when we are done, (c) points at
  // the first non-white character on the line.
  for( cptr = c + strlen( c ) - 1; cptr >= c; cptr-- ) {
	  if( isnewline( *cptr ) )
	    break;
  }
  c = cptr > c ? cptr : c;
  while( isspace( *c ) )
	  c++;
    
  // find the promptstring in the buffer.
  if( dots_wild == 0 ) {
  	// a bit of an obscure case, but: find the _last_ occurrence on the
    // prompt line, not the first.
	  c = strstr( c, promptstring );
	  if( !c ) 
	    return( 0 );
	  else while( strstr( c+1, promptstring ) )
	    c = strstr( c+1, promptstring );
  } else {	// dots are wild, like in regular expressions
   	char *	p 	   = c;
   	unsigned int	char_count = 0;
	  while( *p != '\0' && char_count != strlen( promptstring ) ) {
	    char 	tmp = promptstring[ char_count ];
	    if( tmp == '.' || *p == tmp ) {
	    	if( char_count == 0 ) {
	   	    // If we match the beginning of the promptstring,
	   	    // set c to point at it.
	   	    c = p;
	 	  	}
	    	char_count++;
	    } else {
	    	// Start over, since a letter did not match and was not a dot.
	    	char_count = 0;
	    }
	    p++;
		}

		// If we hit the end of the promptstring, and char_count does not
		// equal its length, then it was not there.
		if( char_count != strlen( promptstring ) )
	    return( 0 );
  }

  // now make sure everything after "promptstring" is whitespace
  // and there is at least one punctuation mark.
  foundpunct = 0;
  for( c += strlen( promptstring ); *c; c++ ) {
    if( strchr( prompt_punct, *c ) )
	    foundpunct = 1;
	  else if( !isspace( *c ) )
      return 0; // non-whitespace or punctuation: not a prompt
  }
  return( foundpunct ); // found a prompt if the string was followed by punct
}

int
AxDialBrain_is_login_prompt( AxDialBrain * brain, char * buf )
{
  return( AxDialBrain_is_prompt( brain, buf, "login",   0 ) ||
    	    AxDialBrain_is_prompt( brain, buf, "name",    0 ) ||
    	    AxDialBrain_is_prompt( brain, buf, "userid",  0 ) ||
    	    AxDialBrain_is_prompt( brain, buf, "user.id", 1 ) ||
    	    AxDialBrain_is_prompt( brain, buf, "signon",  0 ) ||
    	    AxDialBrain_is_prompt( brain, buf, "sign.on", 1 ) ||
    	    AxDialBrain_is_prompt( brain, buf, "usuario", 0 ) ||
    	    ( brain->dialer->options.login_prompt[0] &&
    	      AxDialBrain_is_prompt( brain, buf, brain->dialer->options.login_prompt,0 ) ) );
}

int
AxDialBrain_is_password_prompt( AxDialBrain * brain, char * buf )
/******************************************************/
{
    return( AxDialBrain_is_prompt( brain, buf, "password", 0 ) ||
      ( brain->dialer->options.pass_prompt[0] &&
        AxDialBrain_is_prompt( brain, buf, brain->dialer->options.pass_prompt, 0 ) ) );
}

int
AxDialBrain_is_welcome_msg( AxDialBrain * brain, char * buf )
{
    return(    brain->sent_login
					 && ( strstr( buf, "mtu" ) || strstr( buf, "ip address is" ) ) );
}

BrainToken *
AxDialBrain_tokenize( AxDialBrain * brain, char * left, char * right )
{
    BrainToken *   token_list = NULL;
    BrainToken *   new_token  = NULL;
    BrainToken *   prev_token = NULL;
    char * 	   p;

    if( left == NULL || right == NULL || right <= left )
    	return( NULL );

    p = left;
    while( p <= right ) {
			// If *p is a null or a new-line, we are done.
			if( *p == '\0' || isnewline( *p ) )
	    	break;

			// Skip whitespace in the string.
			if( isspace( *p ) ) {
		    p++;
		    continue;
			}

			// If it's a letter, we've got the beginning of a word.
			if( isalpha( *p ) ) {
	 			char * end = p+1;
		    new_token = (BrainToken*)g_malloc(sizeof(BrainToken));
		    new_token->type = TOK_WORD;
		    new_token->next = NULL;
		    if( token_list == NULL )
		    	token_list = new_token;
		    else
		    	prev_token->next = new_token;
		    while( end <= right && isalpha( *end ) )
		    	end++;
		    new_token->tok_str = (char *)g_malloc( sizeof(char)*(end-p+1) );
		    strncpy( new_token->tok_str, p, end - p );
		    new_token->tok_str[ end-p ] = '\0';
		    p = end;	// skip to the end of the word for next time
		    prev_token = new_token;
		    continue;
			}

			// If it's a digit, we've got the beginning of a number.
			if( isdigit( *p ) ) {
	 	  	char * end = p+1;
	    	new_token = (BrainToken*)g_malloc(sizeof(BrainToken));
	    	new_token->type = TOK_NUMBER;
	    	new_token->next = NULL;
	    	if( token_list == NULL )
	    		token_list = new_token;
	    	else
	    		prev_token->next = new_token;
	    	while( end <= right && isdigit( *end ) )
	    		end++;
	    	new_token->tok_str = (char *)g_malloc( sizeof(char)*(end-p+1) );
	    	strncpy( new_token->tok_str, p, end - p );
	    	new_token->tok_str[ end-p ] = '\0';
	    	p = end;	// skip to the end of the number for next time
	    	prev_token = new_token;
	    	continue;
			}

			// If it's useful punctuation (brackets and such), grab it.
			if( strchr( brackets, *p ) ) {
	    	new_token = (BrainToken*)g_malloc(sizeof(BrainToken));
	    	new_token->type = TOK_PUNCT;
	    	new_token->next = NULL;
	    	if( token_list == NULL )
	    		token_list = new_token;
	    	else
	    		prev_token->next = new_token;
	    	new_token->tok_char = *p;
	    	p++;
	    	prev_token = new_token;
	    	continue;
	 		}

	 		// If it's anything else, ignore it.
	 		p++;
    }

    return( token_list );
}

void
AxDialBrain_token_list_done( AxDialBrain*brain,BrainToken*token_list )
{
    BrainToken * next_token;

    while( token_list != NULL ) {
    	next_token = token_list->next;
    	if( token_list->type == TOK_WORD || token_list->type == TOK_NUMBER )
    	    g_free( token_list->tok_str );
    	g_free( token_list );
    	token_list = next_token;
    }
}

void
AxDialBrain_guess_menu_gets( AxDialBrain*brain,BrainToken*token_list )
{
    int ready_to_break;
    BrainToken *	lmarker = NULL;
    BrainToken *	rmarker = NULL;
    BrainToken *	tok;
    char *		prompt_resp = NULL;
    int			index, failed = 0;

    tok = token_list;
    for( ; tok != NULL; tok = tok->next ) {
    	BrainToken * tok2;
      failed = 0;
    	if( tok->type == TOK_PUNCT )
    	    continue;

    	// Only looking at words and numbers now.
    	tok2 = tok->next;
    	for( ; tok2 != NULL; tok2 = tok2->next ) {
    	   if( tok2->type != TOK_PUNCT )
    	     break;
    	   // Only looking at punctuation after the word we're investigating.
    	   if( strchr( brackets, tok2->tok_char ) ) {
    	   	 failed = 1;
    	   	 break;
    	   }
    	}
    	if( failed )
    	    continue;

    	if( !tok2 || !tok2->next )
    	    break;

    	if( !strcmp( tok2->tok_str, "for" ) &&
    	    !strcmp( tok2->next->tok_str, "ppp" ) )
    	{
    	    AxDialBrain_set_prompt_response( brain, tok->tok_str );
    	    return;
    	}
    }

    tok = token_list;
    for( ; tok != NULL; tok = tok->next )
    	if( tok->type == TOK_PUNCT )
    	    if( strchr( rbrackets, tok->tok_char ) ) {
    	    	rmarker = tok;	// leftmost right-bracket on this line.
    	    	break;
    	    }
    if( rmarker == NULL )
    	return;		// no right-bracket on this line.

    for( tok = rmarker->next; tok != NULL; tok = tok->next )
    	if( tok->type == TOK_WORD && strcmp( tok->tok_str, "ppp" ) == 0 )
    	    break;
    if( tok == NULL )	// We did not find "ppp" after the rmarker
    	return;

    tok = token_list;
    for( ; tok != rmarker; tok = tok->next ) {
    	if( tok->type == TOK_PUNCT ) {
    	    prompt_resp = NULL;
    	    break;
    	}
    	if( tok->type == TOK_NUMBER )
    	    prompt_resp = tok->tok_str;
    	if( tok->type == TOK_WORD )
    	    if( strstr( menu_bad_words, tok->tok_str ) == NULL )
    	    	prompt_resp = tok->tok_str;
    }

    if( prompt_resp != NULL ) {		// Case One was successful!
    	AxDialBrain_set_prompt_response( brain, prompt_resp );
    	return;
    }

    ready_to_break = 0;
    tok     		= token_list;
    rmarker 		= NULL;
    for( ; tok != NULL; tok = tok->next ) {

    	if( tok->type == TOK_PUNCT ) {
    	    if( strchr( lbrackets, tok->tok_char ) ) {
    	    	lmarker = tok;
    	    	ready_to_break = 1;
    	    }
    	} else {
    	    if( ready_to_break )
    	    	break;
    	}
    }
    if( lmarker == NULL )
    	return;		// no left-bracket on this line.

    index = strchr( lbrackets, lmarker->tok_char ) - lbrackets;
    tok = lmarker->next;
    for( ; tok != NULL; tok = tok->next ) {
    	if( tok->type == TOK_PUNCT ) {
    	    if( tok->tok_char == rbrackets[ index ] ) {
    	    	rmarker = tok;
    	    	break;
    	    }
    	} else if( tok->type == TOK_WORD ) {
    	    if( strstr( menu_bad_words, tok->tok_str ) == NULL )
    	    	prompt_resp = tok->tok_str;
    	} else // tok->type == TOK_NUMBER
    	    prompt_resp = tok->tok_str;
    }

    if( rmarker == NULL )
    	return;		// no corresponding right-bracket on this line.

    for( tok = rmarker->next; tok != NULL; tok = tok->next )
    	if( tok->type == TOK_WORD && strcmp( tok->tok_str, "ppp" ) == 0 )
    	    break;
    if( tok == NULL )	// We did not find "ppp" after the rmarker
    	return;

    if( prompt_resp != NULL ) {		// Case Two was successful
    	AxDialBrain_set_prompt_response( brain, prompt_resp );
    	return;
    }

    // Apparently this was not a valid menu option.  Oh well.
    return;
}

void
AxDialBrain_set_prompt_response( AxDialBrain*brain,char*str )
{
    char *	n;

    if( strcmp( str, brain->prompt_response ) ) {
			n = (char*)g_malloc(sizeof(char)*(strlen(str)+1));
			strcpy( n, str );
			n[ strlen( str ) ] = '\0';

//    	syslog( LOG_INFO, "Found a good menu option: \"%s\".\n", n );
    	brain->prompt_response = n;
    }
}
BrainToken *
AxDialBrain_tokenized( AxDialBrain * brain, char * str )
{
  if( str == NULL )
	  return( NULL );

  return( AxDialBrain_tokenize( brain, str, str+strlen( str ) - 1 ) );
}


