/* axdialer.c
 *
 * Copyright (C) 2001 RedHat, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
/*
 * Modified for SIKIGAMI 2001,2002 Axe,Inc.
 */

#include "misc.h"
#include "axconf.h"
#include "axdialer.h"
#include "config.h"
#include "axpapchap.h"

#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <termios.h>
#include <time.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include <assert.h>
#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <memory.h>

static char *	init_responses[] = {
	"ok",
	"error",
	NULL
};

static char *	dial_responses[] = {
	"connect",
	"no carrier",
	"no dialtone",
	"no dial tone",
	"busy",
	"error",
	"voice",
	"fclass",
	NULL
};

static char *	prompt_strings[] = {
	"}!}",
	"!}!",
	NULL
};


void AxDialer_init_valued(AxDialer *dialer,AxConf *cfg,GSList *sections)
{
	AxDialer_init_valued_parm(dialer,cfg,sections,0,NULL);
}
void AxDialer_init_valued_name(AxDialer *dialer,AxConf *cfg,char*name)
{
 GSList * list;
 AxConfigSection * section;

	section = (AxConfigSection*)g_malloc(sizeof(AxConfigSection));
	AxConfigSection_init_valued( section, name );

	list = g_slist_alloc();
	list->data = (gpointer)section;

	AxDialer_init_valued_parm(dialer,cfg,list,0,NULL);
}
void AxDialer_init_valued_parm(AxDialer*dialer,AxConf*_cfg,GSList*_sections,
															 int _chat_mode,char *_remote_name )
{
		GSList * list;

		dialer->cfg				  = _cfg;
    dialer->modem 		  = NULL;
    dialer->ppp_pipe 	  = NULL;
    dialer->been_online = 0;
    dialer->stat 		    = AxDialer_Idle;
    dialer->offset 		  = 0;
    dialer->prompt_tries= 0;
    dialer->last_rx 		= 0;
		dialer->last_execute = 0;
    dialer->prompt_response 	 = "";
    dialer->auto_reconnect_delay = 0;
    dialer->auto_reconnect_at    = 0;
    dialer->connected_at         = 0;
    dialer->weird_pppd_problem   = 0;
    dialer->sect_list = _sections;
    dialer->chat_mode = _chat_mode;
    dialer->remote_name = _remote_name;
    
//    syslog( LOG_INFO, "Internet dialer version " VERSION "\n" );

    // Ensure all sections in sect_list actually exist, warning if not.
		for( list=dialer->sect_list; list; list=g_slist_next(list) ) {
			AxConfigSection * section;
			section = (AxConfigSection*)list->data;
    	if( AxConf_at(dialer->cfg,section->name) == NULL ) {
//		 		syslog(LOG_WARNING,"section [%s] does not exist in sgdial.conf.\n", section->name );
    	}
    }

    // Activate the brain and read configuration.
		dialer->brain = (AxDialBrain*)g_malloc( sizeof(AxDialBrain) );
    AxDialBrain_init_valued(dialer->brain,dialer);

    // init_modem() reads the config options.  It MUST run here!
    
    if( !AxDialer_init_modem(dialer) )
    {
			dialer->stat = AxDialer_ModemError;
			return;
    }
}


void AxDialer_clear( AxDialer * dialer )
{
#if 0
  if( dialer->ppp_pipe ) {
		AxPipe_clear( dialer->ppp_pipe );
		g_free( dialer->ppp_pipe );
	}
  if( dialer->modem ) {
		if ( dialer->modem->isbase ) {
		  AxModemBase_clear( (AxModemBase*)dialer->modem );
		}
		else {
		  AxModem_clear( (AxModem*)dialer->modem );
		}
		g_free( dialer->modem );
	}
#endif
  if( dialer->brain ) {
		AxDialBrain_clear( dialer->brain );
   	g_free( dialer->brain );
	}
  if( dialer->sect_list ) {
		GSList * list;
		for( list=dialer->sect_list; list; list=g_slist_next(list) ) {
			AxConfigSection_clear( (AxConfigSection*)list->data );
			g_free( list->data );
		}
		g_slist_free( dialer->sect_list );
	}
}

int
AxDialer_status( AxDialer * dialer )
{
	return dialer->stat;
}

int
AxDialer_dial( AxDialer * dialer )
{
    if( dialer->stat == AxDialer_Online ) {
    	return( 1 );
		}
	
    if( dialer->stat != AxDialer_Idle ) {
    	return( 0 );
    }

    if( !dialer->options.phnum[0] ) {
    	return( 0 );
//			syslog( LOG_ERR, "Configuration does not specify a valid phone number.\n" );
    	dialer->stat = AxDialer_OtherError;
    }

    if( !dialer->options.login[0] ) {
//			syslog( LOG_ERR, "Configuration does not specify a valid login name.\n" );
    	dialer->stat = AxDialer_OtherError;
    }

    if( !dialer->options.password[0] ) {
//			syslog( LOG_ERR, "Configuration does not specify a valid password.\n" );
    	dialer->stat = AxDialer_OtherError;
    }
    
    if( dialer->stat != AxDialer_Idle ) {
			return( 0 );
		}

    // we need to re-init the modem if we were online before.
    if( dialer->been_online && !AxDialer_init_modem(dialer) ) {
			dialer->stat = AxDialer_ModemError;
		}
    else
    {
			dialer->stat = AxDialer_Dial;
			dialer->connect_attempts = 1;
			dialer->dial_stat = 0;
		  AxDialBrain_reset( dialer->brain );
    }
    
    return( 1 );
}

void
AxDialer_hangup( AxDialer * dialer )
{
  if( dialer->ppp_pipe ) {
		AxPipe_clear( dialer->ppp_pipe );
    g_free( dialer->ppp_pipe );
		dialer->ppp_pipe = NULL;
  }

  if( dialer->stat != AxDialer_Idle ) {
		time_t 	now;
		time( &now );
//		syslog( LOG_INFO, "Disconnecting at %s", ctime( &now ) );
		if( dialer->modem )
		{
	    AxModemBase_hangup( dialer->modem );
			AxModemBase_clear( dialer->modem );
	    g_free( dialer->modem );
	    dialer->modem = NULL;
		}
		dialer->stat = AxDialer_Idle;
  }
}

int
AxDialer_select_setup(AxDialer*dialer,SelectInfo *si)
{
  if( AxDialer_isok(dialer) && dialer->stat != AxDialer_Online && dialer->stat != AxDialer_Idle
    && time( NULL ) - dialer->last_execute > 1 )
  {
	  return( 1 );
  } else {
	  return AxStream_select_setup( (AxStream*)dialer->modem, si );
  }
}

int
AxDialer_isok( AxDialer * dialer )
{
	int b;

	if ( ( ! dialer->modem || AxModem_isok((AxModem*)dialer->modem) )
			&& dialer->stat != AxDialer_ModemError
			&& dialer->stat != AxDialer_OtherError ) {
		b = 1;
	}
	else {
		b = 0;
	}

  if (!b)
	  fprintf(stderr, "Returning not ok!!\n");

  return b;
}

char *
AxDialer_connect_status( AxDialer * dialer )
{
  static char msg[ 160 ];

  switch( dialer->stat ) {
    case AxDialer_PreDial2:
    case AxDialer_PreDial1:
    case AxDialer_Dial:
    case AxDialer_WaitDial:
    	if( dialer->dial_stat == 1 )
    	    strcpy( msg, "Last attempt timed out.  Trying again." );
    	else if( dialer->dial_stat == 2 )
    	    strcpy( msg, "Modem did not connect last attempt.  Trying again." );
    	else if( dialer->dial_stat == 3 )
    	    strcpy( msg, "No dial tone last attempt.  Trying again." );
    	else if( dialer->dial_stat == 4 )
    	    strcpy( msg, "Busy signal on last attempt.  Trying again." );
    	else if( dialer->dial_stat == 5 )
    	    strcpy( msg, "Voice answer on last attempt.  Trying again." );
    	else if( dialer->dial_stat == 6 )
    	    strcpy( msg, "Fax answer on last attempt.  Trying again." );
    	else
    	    return( NULL );
			break;
    case AxDialer_WaitAnything:
    	strcpy( msg, "Waiting for a response from Internet Provider." );
    	break;
    case AxDialer_WaitPrompt:
    	strcpy( msg, "Waiting for a prompt from Internet Provider." );
    	break;
    case AxDialer_AutoReconnectDelay:
    	sprintf( msg, "Next attempt in 00:%02ld:%02ld.", 
    		 ( dialer->auto_reconnect_at - time( NULL ) ) / 60,
    		 ( dialer->auto_reconnect_at - time( NULL ) ) % 60 );
    	break;
    default:
    	return( NULL );
  }
  return( msg );
}

void
AxDialer_execute( AxDialer * dialer )
{
    if( !dialer->modem && !AxDialer_init_modem(dialer) )
    	return;

    dialer->last_execute = time( NULL );
    
    switch( dialer->stat ) {
    	case AxDialer_Dial:
    	case AxDialer_WaitDial:
    	case AxDialer_PreDial1:
    	case AxDialer_PreDial2:
    		AxDialer_async_dial( dialer );
    		break;
   	 	case AxDialer_WaitAnything:
				if( AxStream_select( (AxStream*)dialer->modem, 500, 1, 0, 0 ) ) {
	  		  dialer->stat = AxDialer_WaitPrompt;
	    		dialer->last_rx = time( NULL );
				} else if( time( NULL ) - dialer->last_rx >= 30 ) {
	    		dialer->stat = AxDialer_WaitPrompt;
				} else {
	    		AxStream_write( (AxStream*)dialer->modem, "\r", 1 );
				}
				break;
    	case AxDialer_WaitPrompt:
    		AxDialer_async_waitprompt( dialer );
    		break;
    	case AxDialer_Online:
				assert( !dialer->chat_mode );
				if( dialer->ppp_pipe && AxPipe_child_exited(dialer->ppp_pipe) ) {
	    		if( AxPipe_child_killed(dialer->ppp_pipe) ) {
//						syslog( LOG_ERR, "PPP was killed! (signal = %d)\n",
//		      										AxPipe_exit_status(dialer->ppp_pipe) );
	    		} else {
//						syslog( LOG_ERR, "PPP daemon has died! (exit code = %d)\n",
//		    										  AxPipe_exit_status(dialer->ppp_pipe) );
	   			}
	    
	    		AxDialer_hangup(dialer);
					AxModem_clear( (AxModem*)dialer->modem );
	    		g_free( dialer->modem );
	    		dialer->modem = NULL;
	    
	    		if( time( NULL ) - dialer->connected_at < 45 ) {
						dialer->weird_pppd_problem = 1;
	    		} else {
						dialer->weird_pppd_problem = 0;
	    		}

	    		if( dialer->options.auto_reconnect && AxDialer_isok(dialer) ) {
						if( time( NULL ) - dialer->connected_at >= 45 )
		    			dialer->auto_reconnect_delay = 0;
						dialer->auto_reconnect_delay *= 2;
						if( dialer->auto_reconnect_delay == 0 )
		    			dialer->auto_reconnect_delay = 5;	  // start at 5 seconds
						if( dialer->auto_reconnect_delay > 600 )
		    			dialer->auto_reconnect_delay = 600;  // no longer than 10 minutes

						dialer->auto_reconnect_at = time( NULL ) + dialer->auto_reconnect_delay;

						dialer->stat = AxDialer_AutoReconnectDelay;
//						syslog( LOG_INFO, "Auto Reconnect will be attempted in %d seconds\n", 
//				    									(int)(dialer->auto_reconnect_at-time(NULL)) );
	    		}
				}
				break;
    	case AxDialer_AutoReconnectDelay:
    		if( time( NULL ) >= dialer->auto_reconnect_at ) {
    	    dialer->stat = AxDialer_Idle;
    	    AxDialer_dial(dialer);
    		}
    		break;
    	case AxDialer_Idle:
    	case AxDialer_ModemError:
    	case AxDialer_OtherError:
    	default:
				AxStream_drain( (AxStream*)dialer->modem );
    		break;
    }
}

void
AxDialer_load_options( AxDialer * dialer )
{
    OptInfo opts[] = {
    	{ "Modem",           &dialer->options.modem,        NULL, "/dev/modem",     0 },
    	{ "Init1",           &dialer->options.init1,        NULL, "ATZ",            0 },
    	{ "Init2",           &dialer->options.init2,        NULL, "",               0 },
    	{ "Init3",           &dialer->options.init3,        NULL, "",               0 },
    	{ "Init4",           &dialer->options.init4,        NULL, "",               0 },
    	{ "Init5",           &dialer->options.init5,        NULL, "",               0 },
    	{ "Init6",           &dialer->options.init6,        NULL, "",               0 },
    	{ "Init7",           &dialer->options.init7,        NULL, "",               0 },
    	{ "Init8",           &dialer->options.init8,        NULL, "",               0 },
    	{ "Init9",           &dialer->options.init9,        NULL, "",               0 },
    	{ "Phone",           &dialer->options.phnum,        NULL, "",               0 },
    	{ "Dial Prefix",     &dialer->options.dial_prefix,  NULL, "",               0 },
    	{ "Area Code",       &dialer->options.areacode,     NULL, "",               0 },
    	{ "Dial Command",    &dialer->options.dial_cmd,     NULL, "ATDT",           0 },
    	{ "Username",        &dialer->options.login,        NULL, "",               0 },
    	{ "Login Prompt",    &dialer->options.login_prompt, NULL, "",               0 },
    	{ "Password",        &dialer->options.password,     NULL, "",               0 },
    	{ "Password Prompt", &dialer->options.pass_prompt,  NULL, "",               0 },
    	{ "PPPD Path",       &dialer->options.where_pppd,   NULL, "/usr/sbin/pppd", 0 },
    	{ "Force Address",   &dialer->options.force_addr,   NULL, "",               0 },
    	{ "Remote Name",     &dialer->options.remote,       NULL, "*",              0 },
    	{ "Default Reply",   &dialer->options.default_reply,NULL, "ppp",	    0 },
    	{ "ISDN",	     			 &dialer->options.isdn,	   		 NULL, "",		    0 },

    	{ "Baud",            NULL, &dialer->options.baud,         "", DEFAULT_BAUD },
    	{ "Carrier Check",   NULL, &dialer->options.carrier_check,"", 1         },
    	{ "Stupid Mode",     NULL, &dialer->options.stupid_mode,  "", 0        },
    	{ "New PPPD",	       NULL, &dialer->options.new_pppd, 	   "", 1         },
    	{ "Auto Reconnect",  NULL, &dialer->options.auto_reconnect,"",1	    },
    	{ NULL,		           NULL,                         NULL,  "", 0            }
    };

    char *	d = "Dialer Defaults";
		int i;

    for( i=0; opts[i].name != NULL; i++ ) {
    	if( opts[i].str_member == NULL ) {
    	    *( opts[i].int_member ) =
								AxConf_fuzzy_getint(dialer->cfg,dialer->sect_list,opts[i].name,
		       							AxConf_getint( dialer->cfg, d, opts[i].name, opts[i].int_default ) );
    	} else {
    	     (*opts[i].str_member) = 
    	    			AxConf_fuzzy_get( dialer->cfg, dialer->sect_list, opts[i].name, 
    	    		    			AxConf_defget( dialer->cfg, d, opts[i].name, opts[i].str_default ) );
    	}
    }

    if( dialer->remote_name ) {
			// command line overrides configuration file
			dialer->options.remote = dialer->remote_name;
    }
}

int
AxDialer_init_modem( AxDialer * dialer )
{
    int	received;
    int	init_count;
    
    AxDialer_load_options( dialer );

    if( dialer->modem ) {
			if ( dialer->modem->isbase ) {
			  AxModemBase_clear( (AxModemBase*)dialer->modem );
			}
			else {
			  AxModem_clear( (AxModem*)dialer->modem );
			}
			g_free( dialer->modem );
		}

    if( !dialer->options.modem[0] ) {
//			syslog( LOG_ERR, "Configuration does not specify a valid modem device.\n" );
    	dialer->stat = AxDialer_ModemError;
			return( 0 );
    }

    // Open the modem...
    if( dialer->chat_mode ) {
			dialer->modem = (AxModemBase*)g_malloc( sizeof(AxModemBase) );
			AxModemBase_init_valued( dialer->modem, STDIN_FILENO );
		}
    else {
			dialer->modem = (AxModemBase*)g_malloc( sizeof(AxModem) );
			AxModem_init_valued( (AxModem*)dialer->modem, dialer->options.modem, dialer->options.baud );
		}

    if( !AxModem_isok((AxModem*)dialer->modem) ) {
//			syslog( LOG_ERR, "Cannot open %s: %s\n", dialer->options.modem, strerror(errno) );
			return( 0 );
    }

//    syslog( LOG_INFO, "Initializing modem.\n" );
    
    // make modem happy
    AxStream_print( (AxStream*)dialer->modem, "\r\r\r\r\r" );
    while( AxStream_select( (AxStream*)dialer->modem, 100, 1, 0, 0 ) ) {
			AxStream_drain((AxStream*)dialer->modem);
    }

    // Send up to nine init strings, in order.
    for( init_count=1; init_count<=9; init_count++ ) {
    	char *	this_str;
    	switch( init_count ) {
	    	case 1:    this_str = dialer->options.init1;	break;
	   		case 2:    this_str = dialer->options.init2;	break;
	    	case 3:    this_str = dialer->options.init3;	break;
	    	case 4:    this_str = dialer->options.init4;	break;
    	  case 5:    this_str = dialer->options.init5;	break;
    	  case 6:    this_str = dialer->options.init6;	break;
    	  case 7:    this_str = dialer->options.init7;	break;
    	  case 8:    this_str = dialer->options.init8;	break;
    	  case 9:
        default:
	               this_str = dialer->options.init9;	break;
    	}
    	if( !! *this_str ) {
    	    AxStream_print1( (AxStream*)dialer->modem, "%s\r", this_str );
//    	    syslog( LOG_INFO, "Sending: %s\n", this_str );

    	    received = AxDialer_wait_for_modem( dialer, init_responses, 5000, 1, 1 );
    	    switch( received ) {
    	      case -1:
//    	    		syslog( LOG_ERR, "Modem not responding.\n" );
    	    		return( 0 );
    	    	case 1:
//    	    		syslog( LOG_ERR, "Bad init string.\n" );
    	    		return( 1 );
    	    }
    	}
    }

    // If we're using an ISDN modem, allow one second for the SPID
    // settings to kick in.  It dials so fast anyway that no one will care.
    if( dialer->options.isdn[0] )
			sleep( 1 );
    
    // Everything worked fine.
//    syslog( LOG_INFO, "Modem initialized.\n" );
    return( 1 );
}

void
AxDialer_async_dial( AxDialer * dialer )
{
    int	received;

    if( dialer->stat == AxDialer_PreDial2 ) {
    	// Wait for three seconds and then go to PreDial1.
    	usleep( 3 * 1000 * 1000 );
    	dialer->stat = AxDialer_PreDial1;
    	return;
    }
    
    if( dialer->stat == AxDialer_PreDial1 ) {
	// Hit enter a few times.
			int i;
			for( i=0; i<3; i++ ) {
	    	AxStream_write( (AxStream*)dialer->modem, "\r", 1 );
	    	usleep( 500 * 1000 );
			}
			dialer->stat = AxDialer_Dial;
			return;
    }
	
    if ( dialer->stat == AxDialer_Dial ) {
    	// Construct the dial string.  We use the dial command, prefix,
			// area code, and phone number as specified in the config file.
			char * s = (char *)g_malloc(sizeof(char)*(strlen(dialer->options.dial_cmd)+1
																						 +strlen(dialer->options.dial_prefix)+1
																						 +strlen(dialer->options.areacode)
																						 +strlen(dialer->options.phnum)+2));
			sprintf(s,"%s%s%s%s%s\r", dialer->options.dial_cmd,
				 												dialer->options.dial_prefix,
				 												(  !dialer->options.dial_prefix
																 ||!dialer->options.dial_prefix[0]  ) ? "" : ",",
				 												dialer->options.areacode,
				 												dialer->options.phnum );
			AxStream_print( (AxStream*)dialer->modem, s );
//			syslog( LOG_INFO, "Sending: %s\n", s );
//			syslog( LOG_INFO, "Waiting for carrier.\n" );

			dialer->stat = AxDialer_WaitDial;
			g_free( s );
    }
    received = AxDialer_async_wait_for_modem( dialer, dial_responses, 1, 1 );
    switch( received ) {
    	case -1:	// nothing -- return control.
				if( dialer->last_rx - time( NULL ) >= 60 ) {
//	    		syslog( LOG_WARNING, "Timed out while dialing.  Trying again.\n" );
	    		dialer->stat = AxDialer_PreDial1;
	    		dialer->connect_attempts++;
	    		dialer->dial_stat = 1;
				}
				return;
   		case 0:	// CONNECT
				if( dialer->options.stupid_mode == 1 ) {
//	    		syslog( LOG_INFO,"Carrier detected.  Starting PPP immediately.\n" );
	    		AxDialer_start_ppp(dialer);
				} else {
//	    		syslog( LOG_INFO,"Carrier detected.  Waiting for prompt.\n" );
	    		dialer->stat = AxDialer_WaitAnything;
				}
				return;
    	case 1:	// NO CARRIER
//				syslog( LOG_WARNING, "No Carrier!  Trying again.\n" );
				dialer->stat = AxDialer_PreDial1;
				dialer->connect_attempts++;
				dialer->dial_stat = 2;
				sleep( 2 );
				return;
   		case 2:	// NO DIALTONE
    	case 3:	// NO DIAL TONE
//				syslog( LOG_ERR, "No dial tone.  Trying again in 5 seconds.\n" );
				dialer->stat = AxDialer_PreDial2;
				dialer->connect_attempts++;
				dialer->dial_stat = 3;
				return;
    	case 4:	// BUSY
//				syslog( LOG_ERR, "The line is busy.  Trying again.\n" );
				dialer->stat = AxDialer_PreDial1;
				dialer->connect_attempts++;
				dialer->dial_stat = 4;
				sleep( 2 );
				return;
    	case 5:	// ERROR
//				syslog( LOG_ERR, "Invalid dial command.\n" );
				dialer->stat = AxDialer_ModemError;
				return;
   		case 6:	// VOICE
//  	  	syslog( LOG_ERR, "Voice line detected.  Trying again.\n" );
				dialer->connect_attempts++;
				dialer->dial_stat = 5;
 		   	dialer->stat = AxDialer_PreDial2;
   		 	return;
	    case 7:	// FCLASS
//    		syslog( LOG_ERR, "Fax line detected.  Trying again.\n" );
				dialer->connect_attempts++;
				dialer->dial_stat = 6;
    		dialer->stat = AxDialer_PreDial2;
    		return;
 	   default:
//				syslog( LOG_ERR, "Unknown dial response string.\n" );
				dialer->stat = AxDialer_ModemError;
				return;
    }
}

void
AxDialer_start_ppp( AxDialer * dialer )
{
    char * addr_colon;

    AxPapChap	papchap;
		AxPapChap_init( &papchap );
    AxPapChap_put_secret(&papchap,dialer->options.login,	
																	dialer->options.password,
																	dialer->options.remote );
    if( AxPapChap_isok_pap( &papchap ) == 0 ) {
//    	syslog( LOG_WARNING, "Warning: Could not modify %s: %s\n--> PAP (Password Authentication Protocol) may be flaky.\n",
//    	     			PAP_SECRETS, strerror( errno ) );
    }
    if( AxPapChap_isok_chap( &papchap ) == 0 ) {
//    	syslog( LOG_WARNING,"Warning: Could not modify %s: %s\n--> CHAP (Challenge Handshake) may be flaky.\n",
//					     CHAP_SECRETS, strerror( errno ) );
    }

    if( dialer->chat_mode ) {
			exit(0); // pppd is already started...
		}

    addr_colon = (char*)g_malloc(strlen(dialer->options.force_addr)+2 );
		sprintf( addr_colon, "%s:", dialer->options.force_addr );

	{
    time_t	now;
    char const *argv[] = {
				dialer->options.where_pppd,
				"modem",
				"crtscts",
				"defaultroute",
				"usehostname",
				"-detach",
				"user", dialer->options.login,
				dialer->options.force_addr[0] ? (const char *)addr_colon : "noipdefault",
				dialer->options.remote ? "remotename" : (dialer->options.new_pppd ? "call" : NULL), 
				dialer->options.remote ? (const char*)dialer->options.remote : (dialer->options.new_pppd ? "sgdialer" : NULL),
				(dialer->options.new_pppd && dialer->options.remote) ? "call" : NULL,
				(dialer->options.new_pppd && dialer->options.remote) ? "sgdialer" : NULL,
	NULL
    };

    if( access( dialer->options.where_pppd, X_OK ) != 0 ) {
//      syslog( LOG_ERR, "Unable to run %s.\n", dialer->options.where_pppd );
//      syslog( LOG_ERR, "Check permissions, or specify a \"PPPD Path\" option in sgdial.conf.\n" );
			goto _END_;
    }

    time( &now );
//    syslog( LOG_INFO, "Starting pppd at %s", ctime( &now ) );

    dialer->ppp_pipe = (AxPipe*)g_malloc(sizeof(AxPipe));
		AxPipe_init_valued( dialer->ppp_pipe, argv[0], argv,0,0,0,
			   								dialer->modem->fd, dialer->modem->fd, dialer->modem->fd );

    dialer->stat = AxDialer_Online;
    dialer->been_online  = 1;
    dialer->connected_at = time( NULL );

_END_:
	g_free( addr_colon );
	}
}

void
AxDialer_async_waitprompt( AxDialer * dialer )
{
    int		received;
    char *prompt_response;

    if( dialer->options.carrier_check == 1 ) {
			if( !dialer->modem || !AxModem_carrier((AxModem*)dialer->modem) ) {
	    	dialer->stat = AxDialer_ModemError;
    	    return;
			}
    }

    received = AxDialer_async_wait_for_modem( dialer, prompt_strings, 0, 1 );
    if( received >= 0 ) {
    	// We have a PPP sequence!
//    	syslog( LOG_INFO, "PPP negotiation detected.\n" );
    	AxDialer_start_ppp(dialer);
    } else if( received == -1 ) {
			prompt_response = AxDialBrain_check_prompt( dialer->brain, dialer->buffer );
			if( prompt_response != NULL )
	 		  AxStream_print1( (AxStream*)dialer->modem, "%s\r", prompt_response );
    }
}


static void strip_parity( char * buf, size_t size )
{
    while( size-- > 0 )
    {
			*buf = *buf & 0x7f;
			buf++;
    }
}
char * strlwr( char * string )
{
  char *  p = string;

    while( *p ) {
		  *p = tolower( *p );
		   p++;
		}

    return( string );
}


int
AxDialer_wait_for_modem( AxDialer * dialer,
						char * 	strs[], 
			      int	timeout, 
			      int	neednewline,
			      int	verbose )
{
    off_t	onset;
    char *	soff;
    int		result = -1;
    int		len;
    const char *ppp_marker = NULL;

    while( AxStream_select( (AxStream*)dialer->modem, timeout, 1, 0, 0 ) ) {
			dialer->last_rx = time( NULL );
    	onset = dialer->offset;
			dialer->offset += AxStream_read((AxStream*)dialer->modem,
																			dialer->buffer+dialer->offset,
																			INBUF_SIZE-dialer->offset);
	
			while( dialer->offset < INBUF_SIZE && AxStream_select( (AxStream*)dialer->modem, 100, 1, 0, 0 ) )
	   		dialer->offset += AxStream_read( (AxStream*)dialer->modem, dialer->buffer+dialer->offset,
																INBUF_SIZE-dialer->offset );

			// Make sure there is a NULL on the end of the buffer.
			dialer->buffer[ dialer->offset ] = '\0';

			// Now turn all the NULLs in the middle of the buffer to spaces, for
			// easier parsing.
			replace_char( dialer->buffer+onset,'\0',' ',dialer->offset-onset);
			strip_parity( dialer->buffer+onset, dialer->offset-onset );

			if( verbose )
				fwrite( (void*)(dialer->buffer+onset), dialer->offset-onset, 1, stderr );
//	    	modemrx.write( buffer + onset, offset - onset );

			strlwr( dialer->buffer + onset );

			// Now we can search using strstr.
			for( result = 0; strs[ result ] != NULL; result++ ) {
	    	len = strlen( strs[ result ] );
	    	soff = strstr( dialer->buffer, strs[ result ] );
	    
	    	if( soff && ( !neednewline 
			 		|| strchr( soff, '\n' ) || strchr( soff, '\r' ) ) ) {
					memmove( dialer->buffer,soff+len,dialer->offset-(int)(soff+len-dialer->buffer));
					dialer->offset -= (int)( soff+len - dialer->buffer );
					break;
	    	}
			}
	
			if (strs[ result ] == NULL)
	    	result = -1;

			ppp_marker = AxDialBrain_guess_menu( dialer->brain, dialer->buffer );
			if( ppp_marker != NULL )
	    	memset( dialer->buffer, ' ', ppp_marker-dialer->buffer );
	
			if( dialer->offset == INBUF_SIZE ) {
	    	memmove(dialer->buffer,dialer->buffer+INBUF_SIZE/2,INBUF_SIZE-INBUF_SIZE/2);
	    	dialer->offset = INBUF_SIZE/2;
			}
	
			if (result != -1)
	   		break;
    }
    
    dialer->buffer[ dialer->offset ] = 0;
    return( result ); // -1 == timeout
}

int
AxDialer_async_wait_for_modem( AxDialer * dialer, char * strs[], int neednl, int verbose )
{
    return( AxDialer_wait_for_modem( dialer, strs, 10, neednl, verbose ) );
}

void
AxDialer_reset_offset( AxDialer * dialer )
{
    dialer->offset = 0;
    dialer->buffer[0] = '\0';
}
int
AxDialer_is_pending( AxDialer * dialer )
{
	return AxStream_select( (AxStream*)dialer->modem, 1000, 1, 0, 0);
}

