/* axdialer.h
 *
 * Copyright (C) 2001 RedHat, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
/*
 * Modified for SIKIGAMI 2001,2002 Axe,Inc.
 */

#ifndef __DIALER_H__
#define __DIALER_H__

#include <termios.h>

#include "axconf.h"
#include "axmodem.h"
//#include "axpapchap.h"
#include "axdialbrain.h"
#include "axpipe.h"
//#include "axstreamclone.h"

#define INBUF_SIZE	1024
#define DEFAULT_BAUD	57600U

extern const char sgdial_help_text[];
extern const char sgdial_version_text[];

typedef struct _OptInfo OptInfo;
struct _OptInfo
{
    char *	name;
    char *	* str_member;
    int  *	int_member;
    char *	str_default;
    int		int_default;
};

struct AxConf;

enum {
	AxDialer_Idle = 0,
	AxDialer_ModemError,
	AxDialer_OtherError,
	AxDialer_Online,
	AxDialer_Dial,
	AxDialer_PreDial1,
	AxDialer_PreDial2,
	AxDialer_WaitDial,
	AxDialer_WaitAnything,
	AxDialer_WaitPrompt,
	AxDialer_AutoReconnectDelay
};

struct _AxDialer {
    AxModemBase *modem;

    AxConf		cnf;


    int	   connect_attempts;
    int		 weird_pppd_problem;
    int	   dial_stat;

    struct {
    	char			*	modem;
    	int					baud;
    	char			*	init1;
    	char			* init2;
    	char 			* init3;
    	char			*	init4;
    	char			* init5;
    	char			* init6;
    	char			* init7;
    	char			* init8;
    	char			* init9;
    	char			* phnum;
    	char			* dial_prefix;
    	char			* areacode;
    	char			* dial_cmd;
    	char			* login;
    	char			* login_prompt;
    	char			* password;
    	char			* pass_prompt;
    	char			* where_pppd;
    	char			* force_addr;
    	char			* remote;
    	char			* default_reply;
    	char			* isdn;
    	int		carrier_check;
    	int		stupid_mode;
    	int		new_pppd;
    	int		auto_reconnect;
    } options;

    AxDialBrain *	brain;
    AxConf *	cfg;
    GSList *	sect_list;

    int			chat_mode;
    char 		*	remote_name;
    
		int			modemrx;

    int				been_online;
    time_t		connected_at;
    time_t		auto_reconnect_delay;
    time_t		auto_reconnect_at;
    AxPipe *		ppp_pipe;

    int		stat;

    time_t		last_rx;
    time_t		last_execute;
    int				prompt_tries;
    char			* prompt_response;

    char	buffer[ INBUF_SIZE + 1 ];
    off_t	offset;

};

extern void AxDialer_init_valued(  AxDialer*,AxConf*,GSList*);
extern void AxDialer_init_valued_name(AxDialer*,AxConf*,char*);
extern void AxDialer_init_valued_parm(AxDialer*,AxConf*,GSList*,int,char*);
extern void AxDialer_clear(AxDialer*);

extern int	AxDialer_status( AxDialer * );

extern int		AxDialer_dial( AxDialer * );
extern void		AxDialer_hangup( AxDialer * );
extern int		AxDialer_select_setup( AxDialer *, SelectInfo * );
extern int		AxDialer_isok( AxDialer * );
extern char * AxDialer_connect_status( AxDialer * );
extern void		AxDialer_execute( AxDialer * );

extern void		AxDialer_load_options( AxDialer * );
extern int		AxDialer_init_modem( AxDialer * );

extern void		AxDialer_async_dial( AxDialer * );
extern void		AxDialer_start_ppp( AxDialer * );

extern void		AxDialer_async_waitprompt( AxDialer * );

extern int		AxDialer_wait_for_modem( AxDialer *,char *[], int, int, int );
extern int		AxDialer_async_wait_for_modem( AxDialer *,char * strs[],int neednewline,int);

extern void		AxDialer_reset_offset( AxDialer * );
extern int		AxDialer_is_pending( AxDialer * );

#endif
